"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var path = _interopRequireWildcard(require("path"));
var fs = _interopRequireWildcard(require("fs-extra"));
var _validateNpmPackageName = _interopRequireDefault(require("validate-npm-package-name"));
var _changeCase = require("change-case");
var _prompts = _interopRequireDefault(require("prompts"));
var _npmPacklist = _interopRequireDefault(require("npm-packlist"));
var _arborist = _interopRequireDefault(require("@npmcli/arborist"));
var _getModularRoot = _interopRequireDefault(require("./utils/getModularRoot"));
var _execAsync = _interopRequireDefault(require("./utils/execAsync"));
var logger = _interopRequireWildcard(require("./utils/logger"));
var _actionPreflightCheck = _interopRequireDefault(require("./utils/actionPreflightCheck"));
var _getAllFiles = _interopRequireDefault(require("./utils/getAllFiles"));
var _LineFilterOutStream = _interopRequireDefault(require("./utils/LineFilterOutStream"));
var _parsePackageName = require("./utils/parsePackageName");
var _getWorkspaceInfo = require("./utils/getWorkspaceInfo");
var _isInWorkspaces = require("./utils/isInWorkspaces");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
const packagesRoot = 'packages';
const CUSTOM_TEMPLATE = '__CHOOSE_MY_OWN__';
async function promptForName(name) {
  // The user may try to create a package using a name that already exists
  // so we loop until they choose a unique name or cancel
  while (true) {
    // Default the response the predefined `name` argument
    const response = name ? {
      name
    } : await (0, _prompts.default)({
      name: 'name',
      type: 'text',
      message: `What would you like to name this package?`
    });

    // Exit if the user cancels
    if (!response.name) {
      throw Error('No name entered, exiting');
    }
    return response.name;
  }
}
async function promptForType(type) {
  if (type) {
    return type;
  }
  const response = await (0, _prompts.default)({
    name: 'type',
    type: 'select',
    message: `What kind of package is this?`,
    choices: [{
      title: 'A plain package (package)',
      value: 'package'
    }, {
      title: 'A micro-frontend React view compiled to a bundle file (view)',
      value: 'view'
    }, {
      title: 'A micro-frontend React view compiled to ES Modules (esm-view)',
      value: 'esm-view'
    }, {
      title: 'A micro-frontend React container (app)',
      value: 'app'
    }, {
      title: 'A package containing shared source code which is imported by other packages (source)',
      value: 'source'
    }, {
      title: 'Choose my own template',
      value: CUSTOM_TEMPLATE
    }],
    initial: 0
  });
  if (!response.type) {
    throw Error('No type entered, exiting');
  }
  return response.type;
}
async function promptForTemplate(templateName) {
  if (templateName === CUSTOM_TEMPLATE) {
    logger.warn('Note: you are choosing to install a template that is not maintained by the modular team.');
    const response = await (0, _prompts.default)([{
      name: 'templateName',
      type: 'text',
      message: `Which modular template would you like to use?`
    }]);
    if (!response.templateName) {
      throw Error('No template name entered, exiting');
    }
    if (response.templateName[0] === '@') {
      const [scope, typePackageName] = response.templateName.split('/');
      templateName = `${scope}/modular-template-${typePackageName}`;
    } else {
      templateName = response.templateName;
    }
  }
  if (!templateName.startsWith('modular-template')) {
    return `modular-template-${templateName}`;
  }
  return templateName;
}
async function getYarnVersion() {
  const {
    stdout: version
  } = await (0, _execAsync.default)('yarnpkg', ['--version'], {
    cwd: (0, _getModularRoot.default)(),
    stdout: 'pipe'
  });
  return version;
}
async function addPackage({
  name: nameArg,
  type: typeArg,
  template: templateNameArg,
  preferOffline = true,
  verbose = false,
  unstableName,
  path: pathArg
}) {
  var _modularTemplatePacka, _modularTemplatePacka2, _subprocess$stderr, _rootPackageJson$depe;
  const name = await promptForName(nameArg || unstableName);
  const modularRoot = (0, _getModularRoot.default)();
  const {
    componentName,
    packagePath: newPackagePath
  } = getNewPackageDetails({
    name,
    targetPath: pathArg || path.join(modularRoot, packagesRoot)
  });
  await validatePackageDetails(name, newPackagePath, pathArg);
  const packageType = templateNameArg ?? (await promptForType(typeArg));
  const templateName = await promptForTemplate(templateNameArg || packageType);
  const yarnVersion = await getYarnVersion();
  const isYarnV1 = yarnVersion.startsWith('1.');
  const installedPackageJsonPath = path.join(templateName, 'package.json');

  // Try and find the modular template package. If it's already been installed
  // in the project then continue without needing to do an install.
  // else we will fetch it from the yarn registry.
  let templatePackageJsonPath;
  try {
    logger.log(`Looking for template ${templateName} in project...`);
    templatePackageJsonPath = require.resolve(installedPackageJsonPath, {
      paths: [modularRoot]
    });
  } catch (e) {
    logger.log(`Couldn't find ${templateName} in project`);
    logger.log(`Installing template package ${templateName} from registry, this may take a moment...`);
    const yarnAddArgs = ['add', templateName];
    if (isYarnV1) {
      yarnAddArgs.push('--prefer-offline', '-W');
    } else {
      yarnAddArgs.push('--cached');
    }
    await runYarnCommand(yarnAddArgs, verbose);
    templatePackageJsonPath = require.resolve(installedPackageJsonPath, {
      paths: [modularRoot]
    });
  }
  const modularTemplatePackageJson = await fs.readJSON(templatePackageJsonPath);
  const modularType = modularTemplatePackageJson === null || modularTemplatePackageJson === void 0 ? void 0 : (_modularTemplatePacka = modularTemplatePackageJson.modular) === null || _modularTemplatePacka === void 0 ? void 0 : _modularTemplatePacka.type;
  if (modularType !== 'template') {
    throw new Error(`${templateName} is not marked as a "template". Check the package you are trying to install.`);
  }
  const modularTemplateType = modularTemplatePackageJson === null || modularTemplatePackageJson === void 0 ? void 0 : (_modularTemplatePacka2 = modularTemplatePackageJson.modular) === null || _modularTemplatePacka2 === void 0 ? void 0 : _modularTemplatePacka2.templateType;
  if (!(modularTemplateType === undefined || ['app', 'esm-view', 'view', 'source', 'package'].includes(modularTemplateType))) {
    throw new Error(`${templateName} has modular type: ${modularTemplateType}, which does not exist, please update this template`);
  }
  const templatePath = path.dirname(templatePackageJsonPath);
  // Create new package directory
  await fs.mkdirp(newPackagePath);
  const arborist = new _arborist.default({
    path: templatePath
  });
  const tree = await arborist.loadActual();
  const filesToCopy = await (0, _npmPacklist.default)(tree);
  filesToCopy.forEach(file => {
    if (file !== 'package.json') fs.copySync(path.join(templatePath, file), path.join(newPackagePath, file));
  });
  const packageFilePaths = (0, _getAllFiles.default)(newPackagePath);
  for (const packageFilePath of packageFilePaths) {
    if (/\.(ts|tsx|js|jsx|json|md|txt)$/i.test(packageFilePath)) {
      fs.writeFileSync(packageFilePath, fs.readFileSync(packageFilePath, 'utf8').replace(/PackageName__/g, name).replace(/ComponentName__/g, componentName));
    }
  }
  await fs.writeJson(path.join(newPackagePath, 'package.json'), {
    name,
    private: modularTemplateType === 'app',
    modular: modularTemplateType ? {
      type: modularTemplateType
    } : undefined,
    main: modularTemplatePackageJson === null || modularTemplatePackageJson === void 0 ? void 0 : modularTemplatePackageJson.main,
    dependencies: modularTemplatePackageJson === null || modularTemplatePackageJson === void 0 ? void 0 : modularTemplatePackageJson.dependencies,
    version: '1.0.0'
  }, {
    spaces: 2
  });
  if (modularTemplateType === 'app') {
    // add a tsconfig, because CRA expects it
    await fs.writeJSON(path.join(newPackagePath, 'tsconfig.json'), {
      extends: path.relative(newPackagePath, modularRoot) + '/tsconfig.json'
    }, {
      spaces: 2
    });
  }
  const yarnArgs = [];
  if (isYarnV1) {
    if (verbose) {
      yarnArgs.push('--verbose');
    }
    if (preferOffline) {
      yarnArgs.push('--prefer-offline');
    }
  }
  const subprocess = (0, _execAsync.default)('yarnpkg', yarnArgs, {
    cwd: modularRoot,
    stderr: 'pipe',
    stdout: verbose ? process.stdout : 'ignore'
  });

  // Remove warnings
  (_subprocess$stderr = subprocess.stderr) === null || _subprocess$stderr === void 0 ? void 0 : _subprocess$stderr.pipe(new _LineFilterOutStream.default(/.*warning.*/));
  if (verbose) {
    var _subprocess$stderr2;
    (_subprocess$stderr2 = subprocess.stderr) === null || _subprocess$stderr2 === void 0 ? void 0 : _subprocess$stderr2.pipe(process.stderr);
  }
  await subprocess;

  // Clean up package.json dependencies - the template isn't needed anymore
  const rootPackageJson = fs.readJSONSync(path.join(modularRoot, 'package.json'));
  if ((_rootPackageJson$depe = rootPackageJson.dependencies) !== null && _rootPackageJson$depe !== void 0 && _rootPackageJson$depe[templateName]) {
    const removeTemplateArgs = ['remove', templateName];
    if (isYarnV1) {
      removeTemplateArgs.push('-W');
    }
    await runYarnCommand(removeTemplateArgs, verbose);
  }
}
function getNewPackageDetails({
  name,
  targetPath
}) {
  const {
    module
  } = (0, _parsePackageName.parsePackageName)(name);
  if (!module) {
    throw new Error(`Package name "${name}" is invalid: can't parse module name.`);
  }
  const packageDir = path.join(module);
  const componentName = (0, _changeCase.pascalCase)(module);
  const packagePath = path.resolve(path.join(targetPath, packageDir));
  return {
    componentName,
    packagePath
  };
}
async function validatePackageDetails(name, packagePath, pathArg) {
  // Validate package name
  const {
    validForNewPackages,
    errors
  } = (0, _validateNpmPackageName.default)(name);
  const packageNameFormattedErrors = (errors === null || errors === void 0 ? void 0 : errors.join('\n')) || '';
  if (!validForNewPackages) {
    throw new Error(`Invalid package name "${name}" specified\n${packageNameFormattedErrors}`);
  }

  // Find out if a package with the same name already exists
  if ((await (0, _getWorkspaceInfo.getWorkspaceInfo)())[name]) {
    throw new Error(`A package with name "${name}" already exists`);
  }
  if (pathArg) {
    // Find out if the provided base path is outside of modularRoot
    const relative = path.relative((0, _getModularRoot.default)(), pathArg);
    const isSubdir = relative && !relative.startsWith('..') && !path.isAbsolute(relative);
    if (!isSubdir) {
      throw new Error(`Provided base install path "${pathArg}" is not a descendant of the Modular root directory "${(0, _getModularRoot.default)()}"`);
    }

    // Find out if the provided base path is included in the package.json's "workspaces" globs
    if (!(await (0, _isInWorkspaces.isInWorkspaces)(pathArg))) {
      throw new Error(`Specified package path "${pathArg}" does not match modular workspaces glob patterns`);
    }
  }

  // Find out if the directory already exists and it's not empty
  let dirExists = false;
  try {
    dirExists = !!(await fs.readdir(packagePath)).length;
  } catch {
    // noop: if this throws, the dir doesn't exist
  }
  if (dirExists) {
    throw new Error(`Directory "${packagePath}" already exists and it's not empty`);
  }
}
async function runYarnCommand(args, verbose) {
  var _templateInstallSubpr;
  const templateInstallSubprocess = (0, _execAsync.default)('yarnpkg', args, {
    cwd: (0, _getModularRoot.default)(),
    stderr: 'pipe',
    stdout: 'ignore'
  });

  // Remove warnings
  (_templateInstallSubpr = templateInstallSubprocess.stderr) === null || _templateInstallSubpr === void 0 ? void 0 : _templateInstallSubpr.pipe(new _LineFilterOutStream.default(/.*warning.*/));
  if (verbose) {
    var _templateInstallSubpr2;
    (_templateInstallSubpr2 = templateInstallSubprocess.stderr) === null || _templateInstallSubpr2 === void 0 ? void 0 : _templateInstallSubpr2.pipe(process.stderr);
  }
  await templateInstallSubprocess;
}
var _default = (0, _actionPreflightCheck.default)(addPackage);
exports.default = _default;
//# sourceMappingURL=addPackage.js.map