"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.makeBundle = makeBundle;
var path = _interopRequireWildcard(require("path"));
var _changeCase = require("change-case");
var _builtinModules = _interopRequireDefault(require("builtin-modules"));
var _chalk = _interopRequireDefault(require("chalk"));
var rollup = _interopRequireWildcard(require("rollup"));
var _rollupPluginEsbuild = _interopRequireDefault(require("rollup-plugin-esbuild"));
var _pluginCommonjs = _interopRequireDefault(require("@rollup/plugin-commonjs"));
var _pluginJson = _interopRequireDefault(require("@rollup/plugin-json"));
var _rollupPluginPostcss = _interopRequireDefault(require("rollup-plugin-postcss"));
var _pluginNodeResolve = _interopRequireDefault(require("@rollup/plugin-node-resolve"));
var _getPackageEntryPoints = require("./getPackageEntryPoints");
var _getPrefixedLogger = _interopRequireDefault(require("../../utils/getPrefixedLogger"));
var _getPackageMetadata = _interopRequireDefault(require("../../utils/getPackageMetadata"));
var _getModularRoot = _interopRequireDefault(require("../../utils/getModularRoot"));
var _getRelativeLocation = _interopRequireDefault(require("../../utils/getRelativeLocation"));
var _createEsbuildBrowserslistTarget = _interopRequireDefault(require("../common-scripts/createEsbuildBrowserslistTarget"));
var _config = require("../../utils/config");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
const outputDirectory = 'dist';
const extensions = ['.ts', '.tsx', '.js', '.jsx'];
function distinct(arr) {
  return [...new Set(arr)];
}
async function makeBundle(packageName, preserveModules, includePrivate) {
  const modularRoot = (0, _getModularRoot.default)();
  const metadata = await (0, _getPackageMetadata.default)();
  const {
    rootPackageWorkspaceDefinitions,
    rootPackageJsonDependencies,
    packageJsons,
    packageJsonsByPackagePath,
    packageNames
  } = metadata;
  const paramCaseTarget = (0, _changeCase.paramCase)(packageName);
  const packagePath = await (0, _getRelativeLocation.default)(packageName);
  const targetOutputDirectory = path.join(modularRoot, outputDirectory, paramCaseTarget);
  const logger = (0, _getPrefixedLogger.default)(packageName);
  const packageJson = packageJsonsByPackagePath[packagePath];
  const main = await (0, _getPackageEntryPoints.getPackageEntryPoints)(packagePath, includePrivate);
  logger.log(`building ${packageName}...`);
  const target = (0, _createEsbuildBrowserslistTarget.default)(packagePath);
  const includeDirectories = Array.isArray(rootPackageWorkspaceDefinitions) ? rootPackageWorkspaceDefinitions : (rootPackageWorkspaceDefinitions === null || rootPackageWorkspaceDefinitions === void 0 ? void 0 : rootPackageWorkspaceDefinitions.packages) ?? [];
  const bundle = await rollup.rollup({
    input: path.join(modularRoot, packagePath, main),
    external: id => {
      // via tsdx
      // TODO: this should probably be included into deps instead
      if (id === 'babel-plugin-transform-async-to-promises/helpers') {
        // we want to inline these helpers
        return false;
      }
      // exclude any dependency that's not a realtive import
      return !id.startsWith('.') && !path.isAbsolute(id);
    },
    treeshake: {
      // via tsdx: Don't use getters and setters on plain objects.
      propertyReadSideEffects: false
    },
    plugins: [(0, _pluginNodeResolve.default)({
      extensions,
      browser: true,
      mainFields: ['module', 'main', 'browser']
    }), (0, _pluginCommonjs.default)({
      include: /\/node_modules\//
    }), (0, _rollupPluginEsbuild.default)({
      target,
      minify: false,
      include: includeDirectories,
      exclude: 'node_modules/**'
    }), (0, _rollupPluginPostcss.default)({
      extract: false
    }),
    // TODO: add sass, dotenv
    (0, _pluginJson.default)()]
    // TODO: support for css modules, sass, dotenv,
    // and anything else create-react-app supports
    // (alternatively, disable support for those in apps)
  });

  const absolutePackagePath = path.join(modularRoot, packagePath);
  const outputOptions = {
    freeze: false,
    sourcemap: (0, _config.getConfig)('generateSourceMap', absolutePackagePath),
    sourcemapPathTransform(relativeSourcePath, sourceMapPath) {
      // make source map input files relative to the `${packagePath}/dist-${format}` within
      // the package directory

      const absoluteSourcepath = path.resolve(path.dirname(sourceMapPath), relativeSourcePath);
      const packageRelativeSourcePath = path.relative(absolutePackagePath, absoluteSourcepath);
      return `../${packageRelativeSourcePath}`;
    }
  };

  // we're going to use bundle.write() to actually generate the
  // output files, but first we're going to do a scan
  // to validate dependencies and collect some metadata for later
  const {
    output
  } = await bundle.generate(outputOptions);
  // TODO: we should use this loop to generate the files itself
  // to avoid the second scan, but it's ok for now I guess.
  // "local" workspaces/packages that were imported, i.e - packages/*
  const localImports = {};

  // this is used to collect local filenames being referenced
  // to prevent errors where facades are imported as dependencies
  // and are collected in missingDependencies
  const localFileNames = new Set();

  // imports that aren't defined in package.json or root package.json
  // Now, this will also mark dependencies that were transient/nested,
  // but I think that's the right choice; a dependency might remove it,
  // even in a patch, and it'll break your code and you wouldn't know why.
  const missingDependencies = new Set();

  // get all the names of the files we outputted to make sure they're included
  // in the missing dep check
  const chunkOrAssetFileNames = new Set(output.map(chunkOfAsset => chunkOfAsset.fileName));
  for (const chunkOrAsset of output) {
    if (chunkOrAsset.type === 'asset') {
      // TODO: what should happen here?
    } else {
      // it's a 'chunk' of source code, let's analyse it
      for (const imported of [...chunkOrAsset.imports, ...chunkOrAsset.dynamicImports]) {
        // get the dependency (without references any inner modules)
        const importedPath = imported.split('/');
        const importedPackage =
        // scoped package?
        importedPath[0][0] === '@' ? `${importedPath[0]}/${importedPath[1]}` :
        // non-scoped
        importedPath[0];
        if (importedPackage !== imported && packageNames.includes(importedPackage) &&
        // it's fine if it's anything but a js file
        extensions.includes(path.extname(imported))) {
          // TODO: revisit this if and when we have support for multiple entrypoints
          // TODO: add a line number and file name here
          logger.error(`cannot import a submodule ${imported} from ${importedPackage}`);
          // TODO: This could probably be an error, but
          // let's revisit it when we have a better story.
        }

        if (packageJsons[importedPackage]) {
          // This means we're importing from a local workspace
          // Let's collect the name and add it in the package.json
          // we publish to the registry
          // TODO: make sure local workspaces are NOT explicitly included in package.json
          if (includePrivate || packageJsons[importedPackage].private !== true) {
            localImports[importedPackage] = packageJsons[importedPackage].version;
          } else {
            throw new Error(`referencing a private package: ${importedPackage}`); // TODO - lineNo, filename
          }
        } else {
          var _packageJson$dependen, _packageJson$peerDepe;
          // remote
          if (
          // not mentioned in the local package.json
          !((_packageJson$dependen = packageJson.dependencies) !== null && _packageJson$dependen !== void 0 && _packageJson$dependen[importedPackage]) && !((_packageJson$peerDepe = packageJson.peerDependencies) !== null && _packageJson$peerDepe !== void 0 && _packageJson$peerDepe[importedPackage])) {
            if (rootPackageJsonDependencies[importedPackage]) {
              localImports[importedPackage] = rootPackageJsonDependencies[importedPackage];
            } else {
              // not mentioned in the root package.json either, so
              // let's collect its name and throw an error later
              // TODO: if it's in root's dev dependencies, should throw a
              // different kind of error
              if (!_builtinModules.default.includes(importedPackage) &&
              // In the case that the importedPackage is contained in the
              // files outputted by the bundle write then the import name
              // is a dynamic import which generated a file split. This is
              // perfectly file since we know the file exists from the bundle
              // write phase.
              !chunkOrAssetFileNames.has(importedPackage)) {
                // save filename to remove from missingDeps later
                // if they exist there
                localFileNames.add(chunkOrAsset.fileName);
                missingDependencies.add(importedPackage);
              }
            }
          }
        }
      }
    }
  }
  if (Object.keys(localImports).length > 0) {
    logger.log('Adding dependencies to the generated package.json:');
    Object.entries(localImports).forEach(([packageName, packageVersion]) => {
      logger.log(`\t${packageName}: ${_chalk.default.green(packageVersion)}`);
    });
  }

  // remove local filenames from missingDependencies
  const missingDependenciesWithoutLocalFileNames = [...missingDependencies].filter(dep => !localFileNames.has(dep));
  if (missingDependenciesWithoutLocalFileNames.length > 0) {
    missingDependenciesWithoutLocalFileNames.forEach(missingImport => {
      logger.error(`  ${missingImport}`);
    });
    throw new Error(`Missing dependencies found.`);
  }

  // now actually write the bundles to disk
  // TODO: write to disk in the above check itself to prevent this 2nd pass
  const {
    output: buildOutput
  } = await bundle.write({
    ...outputOptions,
    ...(preserveModules ? {
      preserveModules: true,
      dir: path.join(targetOutputDirectory, `${outputDirectory}-cjs`)
    } : {
      file: path.join(targetOutputDirectory, `${outputDirectory}-cjs`, paramCaseTarget + '.cjs.js')
    }),
    format: 'cjs',
    exports: 'auto'
  });
  await bundle.write({
    ...outputOptions,
    ...(preserveModules ? {
      preserveModules: true,
      dir: path.join(targetOutputDirectory, `${outputDirectory}-es`)
    } : {
      file: path.join(targetOutputDirectory, `${outputDirectory}-es`, paramCaseTarget + '.es.js')
    }),
    format: 'es',
    exports: 'auto'
  });
  const outputPath = buildOutput[0].fileName;

  // return the public facing package.json that we'll write to disk later
  return {
    ...packageJson,
    main: preserveModules ? path.posix.join(`${outputDirectory}-cjs`, outputPath) : `${outputDirectory}-cjs/${paramCaseTarget + '.cjs.js'}`,
    module: preserveModules ? path.posix.join(`${outputDirectory}-es`, outputPath) : `${outputDirectory}-es/${paramCaseTarget + '.es.js'}`,
    typings: path.posix.join(`${outputDirectory}-types`, path.posix.relative('src', main).replace(/\.tsx?$/, '.d.ts')),
    dependencies: {
      ...packageJson.dependencies,
      ...localImports
    },
    /**
     * Certain files are always included, regardless of settings.
     * https://docs.npmjs.com/cli/v9/configuring-npm/package-json#files
     */
    files: distinct([...(packageJson.files || []), 'dist-cjs', 'dist-es', 'dist-types'])
  };
}
//# sourceMappingURL=makeBundle.js.map