"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buildStandalone = buildStandalone;
var _path = _interopRequireDefault(require("path"));
var _fsExtra = _interopRequireDefault(require("fs-extra"));
var _chalk = _interopRequireDefault(require("chalk"));
var logger = _interopRequireWildcard(require("../utils/logger"));
var _getLocation = _interopRequireDefault(require("../utils/getLocation"));
var _determineTargetPaths = _interopRequireDefault(require("./common-scripts/determineTargetPaths"));
var _printHostingInstructions = _interopRequireDefault(require("./printHostingInstructions"));
var _fileSizeReporter = require("./fileSizeReporter");
var _checkBrowsers = require("../utils/checkBrowsers");
var _checkRequiredFiles = _interopRequireDefault(require("../utils/checkRequiredFiles"));
var _createEsbuildBrowserslistTarget = _interopRequireDefault(require("./common-scripts/createEsbuildBrowserslistTarget"));
var _api = require("./esbuild-scripts/api");
var _webpackFileSizeReporter = require("./webpackFileSizeReporter");
var _esbuildFileSizeReporter = require("./esbuildFileSizeReporter");
var _getDependencyInfo = require("../utils/getDependencyInfo");
var _isReactNewApi = require("../utils/isReactNewApi");
var _config = require("../utils/config");
var _buildWebpack = _interopRequireDefault(require("./webpack-scripts/buildWebpack"));
var _getModularRoot = _interopRequireDefault(require("../utils/getModularRoot"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
async function buildStandalone(target, type) {
  // Setup Paths
  const targetDirectory = await (0, _getLocation.default)(target);
  const paths = (0, _determineTargetPaths.default)(target, targetDirectory);
  const isApp = type === 'app';
  const isEsbuild = (0, _config.getConfig)('useModularEsbuild', targetDirectory);
  await (0, _checkBrowsers.checkBrowsers)(targetDirectory);
  let previousFileSizes;
  if (isEsbuild) {
    previousFileSizes = await (0, _esbuildFileSizeReporter.esbuildMeasureFileSizesBeforeBuild)(paths.appBuild);
  } else {
    previousFileSizes = await (0, _webpackFileSizeReporter.webpackMeasureFileSizesBeforeBuild)(paths.appBuild);
  }

  // Warn and crash if required files are missing
  isApp ? await (0, _checkRequiredFiles.default)([paths.appHtml, paths.appIndexJs]) : await (0, _checkRequiredFiles.default)([paths.appIndexJs]);
  logger.log('Creating an optimized production build...');

  // Copy assets from public/ to dist/, execept for index.html
  await _fsExtra.default.emptyDir(paths.appBuild);
  if (await _fsExtra.default.pathExists(paths.appPublic)) {
    await _fsExtra.default.copy(paths.appPublic, paths.appBuild, {
      dereference: true,
      filter: file => file !== paths.appHtml,
      overwrite: true
    });
  }
  let assets;
  logger.debug('Extracting dependency info from source code...');

  // Retrieve dependency info for target to inform the build process
  const {
    importInfo,
    styleImports,
    packageDependencies,
    bundledDependencies,
    bundledResolutions,
    externalDependencies,
    externalResolutions
  } = await (0, _getDependencyInfo.getDependencyInfo)(target);
  logger.debug(`These are the external dependencies and their resolutions: ${JSON.stringify({
    externalDependencies,
    externalResolutions
  })}`);
  logger.debug(`These are the bundled dependencies and their resolutions: ${JSON.stringify({
    bundledDependencies,
    bundledResolutions
  })}`);
  const useReactCreateRoot = (0, _isReactNewApi.isReactNewApi)(externalResolutions);

  // If it's an app, set it at ESBUILD_TARGET_FACTORY or default to es2015
  // If it's not an app it's an ESM view, then we need es2020
  const browserTarget = (0, _createEsbuildBrowserslistTarget.default)(targetDirectory);
  const esbuildTargetFactory = isApp ? browserTarget : ['es2020'];
  let jsEntryPoint;
  let cssEntryPoint;
  if (isEsbuild) {
    logger.debug('Building with esbuild');
    const {
      default: buildEsbuildApp
    } = await Promise.resolve().then(() => _interopRequireWildcard(require('./esbuild-scripts/build')));
    const result = await buildEsbuildApp(target, paths, importInfo, type);
    jsEntryPoint = (0, _api.getEntryPoint)(paths, result, '.js');
    cssEntryPoint = (0, _api.getEntryPoint)(paths, result, '.css');
    assets = (0, _esbuildFileSizeReporter.createEsbuildAssets)(result);
  } else {
    var _stats$assetsByChunkN, _stats$warnings;
    logger.debug('Building with Webpack');
    // create-react-app doesn't support plain module outputs yet,
    // so --preserve-modules has no effect here

    const stats = await (0, _buildWebpack.default)(target, esbuildTargetFactory, isApp, importInfo, useReactCreateRoot, styleImports, paths);
    const mainEntrypoint = (_stats$assetsByChunkN = stats.assetsByChunkName) === null || _stats$assetsByChunkN === void 0 ? void 0 : _stats$assetsByChunkN.main;
    jsEntryPoint = mainEntrypoint === null || mainEntrypoint === void 0 ? void 0 : mainEntrypoint.find(entryPoint => entryPoint.endsWith('.js'));
    cssEntryPoint = mainEntrypoint === null || mainEntrypoint === void 0 ? void 0 : mainEntrypoint.find(entryPoint => entryPoint.endsWith('.css'));
    if ((_stats$warnings = stats.warnings) !== null && _stats$warnings !== void 0 && _stats$warnings.length) {
      logger.log(_chalk.default.yellow('Compiled with warnings.\n'));
      logger.log(stats.warnings.join('\n\n'));
      logger.log('\nSearch for the ' + _chalk.default.underline(_chalk.default.yellow('keywords')) + ' to learn more about each warning.');
    } else {
      logger.log(_chalk.default.green('Compiled successfully.\n'));
    }
    assets = (0, _webpackFileSizeReporter.createWebpackAssets)(paths, stats);
  }
  if (!jsEntryPoint) {
    throw new Error("Can't find main entrypoint after building");
  }

  // Only Webpack apps create their index files separately with a plugin - exclude them
  if (!isApp || isEsbuild) {
    await (0, _api.writeOutputIndexFiles)({
      paths,
      cssEntryPoint,
      jsEntryPoint,
      styleImports,
      importInfo,
      modularType: type,
      externalResolutions
    });
  }

  // Add dependencies from source and bundled dependencies to target package.json
  const targetPackageJson = await _fsExtra.default.readJSON(_path.default.join(targetDirectory, 'package.json'));
  const rootPackageJson = await _fsExtra.default.readJSON(_path.default.join((0, _getModularRoot.default)(), 'package.json'));

  // Copy selected fields of package.json over
  await _fsExtra.default.writeJson(_path.default.join(paths.appBuild, 'package.json'), {
    name: targetPackageJson.name,
    version: targetPackageJson.version,
    license: targetPackageJson.license,
    modular: targetPackageJson.modular,
    dependencies: packageDependencies,
    bundledDependencies: isApp ? Object.keys(packageDependencies) : Object.keys(bundledResolutions),
    module: jsEntryPoint ? paths.publicUrlOrPath + jsEntryPoint : undefined,
    style: cssEntryPoint ? paths.publicUrlOrPath + cssEntryPoint : undefined,
    styleImports: styleImports !== null && styleImports !== void 0 && styleImports.size ? [...styleImports] : undefined,
    engines: targetPackageJson.engines ?? rootPackageJson.engines
  }, {
    spaces: 2
  });
  (0, _fileSizeReporter.printFileSizesAfterBuild)(assets, previousFileSizes);
  (0, _printHostingInstructions.default)(target, paths.publicUrlOrPath, paths.publicUrlOrPath, paths.appBuild);
}
//# sourceMappingURL=buildStandalone.js.map