"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getHttpsConfig;
exports.readEnvFile = readEnvFile;
exports.validateKeyAndCerts = validateKeyAndCerts;
var _fs = _interopRequireDefault(require("fs"));
var _path = _interopRequireDefault(require("path"));
var _crypto = _interopRequireDefault(require("crypto"));
var _chalk = _interopRequireDefault(require("chalk"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// Ensure the certificate and key provided are valid and if not
// throw an easy to debug error
function validateKeyAndCerts({
  cert,
  key,
  keyPath,
  certPath
}) {
  let encrypted;
  try {
    // publicEncrypt will throw an error with an invalid cert
    encrypted = _crypto.default.publicEncrypt(cert, Buffer.from('test'));
  } catch (err) {
    if (err instanceof Error) {
      throw new Error(`The certificate "${_chalk.default.yellow(certPath)}" is invalid.\n${err.message}`);
    }
  }
  if (encrypted) {
    try {
      // privateDecrypt will throw an error with an invalid key
      _crypto.default.privateDecrypt(key, encrypted);
    } catch (err) {
      if (err instanceof Error) {
        throw new Error(`The certificate key "${_chalk.default.yellow(keyPath)}" is invalid.\n${err.message}`);
      }
    }
  }
}

// Read file and throw an error if it doesn't exist
function readEnvFile(file, type) {
  if (!_fs.default.existsSync(file)) {
    throw new Error(`You specified ${_chalk.default.cyan(type)} in your env, but the file "${_chalk.default.yellow(file)}" can't be found.`);
  }
  return _fs.default.readFileSync(file);
}

// Get the https config
// Return cert files if provided in env, otherwise just true or false
function getHttpsConfig(paths, modularRoot) {
  const {
    SSL_CRT_FILE,
    SSL_KEY_FILE,
    HTTPS
  } = process.env;
  const isHttps = HTTPS === 'true';
  let cert;
  let key;
  let keyPath;
  let certPath;
  if (isHttps && SSL_CRT_FILE && SSL_KEY_FILE) {
    // 1. Look in the app directory (non-root) - legacy behaviour
    certPath = _path.default.resolve(paths.appPath, SSL_CRT_FILE);
    keyPath = _path.default.resolve(paths.appPath, SSL_KEY_FILE);
    try {
      cert = readEnvFile(certPath, 'SSL_CRT_FILE');
      key = readEnvFile(keyPath, 'SSL_KEY_FILE');
    } catch (e) {
      // 2. Fall back to the modular root
      certPath = _path.default.resolve(modularRoot, SSL_CRT_FILE);
      keyPath = _path.default.resolve(modularRoot, SSL_KEY_FILE);
      cert = readEnvFile(certPath, 'SSL_CRT_FILE');
      key = readEnvFile(keyPath, 'SSL_KEY_FILE');
    }
    const config = {
      cert,
      key
    };
    validateKeyAndCerts({
      ...config,
      keyPath,
      certPath
    });
    return config;
  }
  return isHttps;
}
//# sourceMappingURL=getHttpsConfig.js.map