"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createStartIndex = createStartIndex;
exports.createViewTrampoline = createViewTrampoline;
exports.getEntryPoint = getEntryPoint;
exports.writeOutputIndexFiles = writeOutputIndexFiles;
var path = _interopRequireWildcard(require("path"));
var fs = _interopRequireWildcard(require("fs-extra"));
var parse5 = _interopRequireWildcard(require("parse5"));
var _semver = _interopRequireDefault(require("semver"));
var _dedent = _interopRequireDefault(require("dedent"));
var _escapeStringRegexp = _interopRequireDefault(require("escape-string-regexp"));
var _htmlMinifierTerser = _interopRequireDefault(require("html-minifier-terser"));
var _getModularRoot = _interopRequireDefault(require("../../utils/getModularRoot"));
var _formatPath = require("./utils/formatPath");
var _getClientEnvironment = _interopRequireDefault(require("../common-scripts/getClientEnvironment"));
var _importInfo = require("../../utils/importInfo");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
const indexFileTemplate = (0, _dedent.default)(`
<!DOCTYPE html>
<html>
  <body>
    <div id="root"></div>
  </body>
</html>
`);
/**
 * @typedef {Object} WriteFilesArguments
 * @property {Paths} paths - list of paths relative to your application or esm-view
 * @property {string} cssEntryPoint - the name of your css entrypoint file
 * @property {string} jsEntryPoint - the name of your js entrypoint file
 * @property {Set<string>} styleImports - a set containing global style import URLs
 * @property {Map<string, string>} importInfo - a map from package name to CDN URL
 * @property {Dependency} externalResolutions - a record of external resolutions and their versions
 * @property {Extract<ModularType, 'app' | 'esm-view'>;} modularType - Modular type, can be "app" or "esm-view"
 */

/**
 * Write `index.html` in the `appBuild` directory specified in `paths`,
 * linking the provided entrypoints (`jsEntryPoint` and `cssEntryPoint`) into it;
 * optionally write a `/static/js/_trampoline.js` trampoline file if `modularType` is "esm-view"
 * and link it in the `index.html`, along with the global `styleImports`.
 * The trampoline file is compatible with the version of React specified in `externalResolutions`.
 * @param  {WriteFilesArguments} arguments - a {@link WriteFilesArguments} object
 * @return {Promise<void>}
 */

async function writeOutputIndexFiles({
  paths,
  cssEntryPoint,
  jsEntryPoint,
  styleImports,
  importInfo,
  externalResolutions,
  modularType
}) {
  const indexContent = fs.existsSync(paths.appHtml) ? await fs.readFile(paths.appHtml, {
    encoding: 'utf-8'
  }) : indexFileTemplate;
  const env = (0, _getClientEnvironment.default)(paths.publicUrlOrPath.slice(0, -1));
  const indexConfiguration = {
    indexContent,
    cssEntryPoint,
    jsEntryPoint,
    styleImports,
    includeTrampoline: modularType === 'esm-view',
    includeRuntime: false,
    replacements: env.raw
  };
  const html = compileIndex(indexConfiguration);
  const minifiedHtml = await _htmlMinifierTerser.default.minify(html, {
    html5: true,
    collapseBooleanAttributes: true,
    collapseWhitespace: true,
    collapseInlineTagWhitespace: true,
    decodeEntities: true,
    minifyCSS: true,
    minifyJS: true,
    removeAttributeQuotes: false,
    removeComments: true,
    removeTagWhitespace: true
  });
  await fs.writeFile(path.join(paths.appBuild, 'index.html'), minifiedHtml);
  if (modularType === 'esm-view') {
    const reactVersion = externalResolutions === null || externalResolutions === void 0 ? void 0 : externalResolutions['react'];
    const useReactCreateRoot = Boolean(reactVersion && _semver.default.gte(reactVersion, '18.0.0'));
    const trampolineContent = createViewTrampoline({
      fileName: path.basename(jsEntryPoint),
      importInfo,
      useReactCreateRoot
    });
    const trampolinePath = `${paths.appBuild}/static/js/_trampoline.js`;
    await fs.writeFile(trampolinePath, trampolineContent);
  }
}
async function createStartIndex({
  paths,
  metafile,
  replacements,
  styleImports,
  isApp
}) {
  const indexContent = (await fs.pathExists(paths.appHtml)) ? await fs.readFile(paths.appHtml, {
    encoding: 'utf-8'
  }) : indexFileTemplate;
  const cssEntryPoint = metafile ? (0, _formatPath.normalizeToPosix)(getEntryPoint(paths, metafile, '.css')) : undefined;
  const jsEntryPoint = metafile ? (0, _formatPath.normalizeToPosix)(getEntryPoint(paths, metafile, '.js')) : undefined;
  const configuration = {
    indexContent,
    cssEntryPoint,
    jsEntryPoint,
    replacements,
    includeRuntime: true,
    includeTrampoline: !isApp,
    styleImports
  };
  return compileIndex(configuration);
}
function createViewTrampoline({
  fileName,
  importInfo,
  useReactCreateRoot
}) {
  const fileRelativePath = `./${fileName}`;
  const reactDomCdnLocation = (0, _importInfo.rewriteModuleSpecifier)(importInfo, 'react-dom');
  const reactCdnLocation = (0, _importInfo.rewriteModuleSpecifier)(importInfo, 'react');
  if (!reactCdnLocation || !reactDomCdnLocation) {
    throw new Error(`react (${reactCdnLocation ?? 'undefined'}) or react-dom (${reactDomCdnLocation ?? 'undefined'}) location not specified. are you sure you have them in your package.json?`);
  }
  return useReactCreateRoot ?
  // use the new createRoot API with React >= 18. The old one is backwards compatible, but will warn on develop.
  (0, _dedent.default)(`import { createRoot } from "${reactDomCdnLocation}/client";
              import React from "${reactCdnLocation}";
              import Component from "${fileRelativePath}";
              var container = document.getElementById("root");
              var root = createRoot(container);
              root.render(React.createElement(Component, null));`) :
  // use the old ReactDOM.render API with React < 18.
  (0, _dedent.default)(`import ReactDOM from "${reactDomCdnLocation}";
              import React from "${reactCdnLocation}";
              import Component from "${fileRelativePath}";
              var DOMRoot = document.getElementById("root");
              ReactDOM.render(React.createElement(Component, null), DOMRoot);`);
}
function getEntryPoint(paths, metafile, type) {
  const result = Object.entries(metafile.outputs).find(([key, output]) => {
    return output.entryPoint && path.extname(key) === type;
  });
  const outputFileName = result === null || result === void 0 ? void 0 : result[0];
  if (outputFileName) {
    return path.relative(paths.appBuild, path.join((0, _getModularRoot.default)(), outputFileName));
  } else {
    return undefined;
  }
}
function compileIndex({
  indexContent,
  cssEntryPoint,
  jsEntryPoint,
  replacements,
  includeRuntime,
  includeTrampoline,
  styleImports
}) {
  const page = parse5.parse(indexContent);
  const html = page.childNodes.find(node => node.nodeName === 'html');
  const head = html.childNodes.find(node => node.nodeName === 'head');
  const body = html.childNodes.find(node => node.nodeName === 'body');
  if (styleImports) {
    [...styleImports].forEach(importUrl => head.childNodes.push(...parse5.parseFragment(`<link rel="stylesheet" href="${importUrl}" />`).childNodes));
  }
  if (cssEntryPoint) {
    head.childNodes.push(...parse5.parseFragment(`<link rel="stylesheet" href="%PUBLIC_URL%/${cssEntryPoint}"></script>`).childNodes);
  }
  if (jsEntryPoint) {
    body.childNodes.push(...parse5.parseFragment(`<script type="module" src="%PUBLIC_URL%/${jsEntryPoint}"></script>`).childNodes);
  }
  if (includeTrampoline) {
    body.childNodes.push(...parse5.parseFragment(`<script type="module" src="%PUBLIC_URL%/static/js/_trampoline.js"></script>`).childNodes);
  }
  if (includeRuntime) {
    body.childNodes.push(...parse5.parseFragment(`<script type="module" src="%PUBLIC_URL%/_runtime/index.js"></script>`).childNodes);
  }
  let data = parse5.serialize(page);

  // Run HTML through a series of user-specified string replacements.
  Object.keys(replacements).forEach(key => {
    const value = replacements[key];
    data = data.replace(new RegExp('%' + (0, _escapeStringRegexp.default)(key) + '%', 'g'), value);
  });
  return data;
}
//# sourceMappingURL=api.js.map