"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.canReadAsset = canReadAsset;
exports.printFileSizesAfterBuild = printFileSizesAfterBuild;
var path = _interopRequireWildcard(require("path"));
var _chalk = _interopRequireDefault(require("chalk"));
var _filesize = _interopRequireDefault(require("filesize"));
var _stripAnsi = _interopRequireDefault(require("strip-ansi"));
var logger = _interopRequireWildcard(require("../utils/logger"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
// These sizes are pretty large. We'll warn for bundles exceeding them.
const WARN_AFTER_BUNDLE_GZIP_SIZE = 512 * 1024;
const WARN_AFTER_CHUNK_GZIP_SIZE = 1024 * 1024;

// Input: 1024, 2048
// Output: "(+1 KB)"
const FIFTY_KILOBYTES = 1024 * 50;

// Prints a detailed summary of build files.
function printFileSizesAfterBuild(assets, previousSizeMap) {
  const sizedAssets = assets.sort((a, b) => b.size - a.size).map(asset => {
    const previousSize = previousSizeMap[asset.normalizedName];
    const differenceLabel = getDifferenceLabel(asset.size, previousSize);
    return {
      ...asset,
      sizeLabel: (0, _filesize.default)(asset.size),
      differenceLabel
    };
  });
  const longestSizeLabelLength = Math.max.apply(null, sizedAssets.map(a => (0, _stripAnsi.default)(a.sizeLabel).length));
  const longestDifferenceLabelLength = Math.max.apply(null, sizedAssets.map(a => (0, _stripAnsi.default)(a.differenceLabel).length));
  const suggestBundleSplitting = false;
  logger.log('File sizes after gzip:');
  logger.log();
  sizedAssets.forEach(asset => {
    let sizeLabel = asset.sizeLabel;
    const sizeLength = (0, _stripAnsi.default)(sizeLabel).length;
    if (sizeLength < longestSizeLabelLength) {
      const rightPadding = ' '.repeat(longestSizeLabelLength - sizeLength);
      sizeLabel += rightPadding;
    }
    let differenceLabel = asset.differenceLabel;
    const differenceLength = (0, _stripAnsi.default)(differenceLabel).length;
    if (differenceLength < longestDifferenceLabelLength) {
      const rightPadding = ' '.repeat(longestDifferenceLabelLength - differenceLength);
      differenceLabel += rightPadding;
    }
    const isMainBundle = asset.name.indexOf('main.') === 0;
    const maxRecommendedSize = isMainBundle ? WARN_AFTER_BUNDLE_GZIP_SIZE : WARN_AFTER_CHUNK_GZIP_SIZE;
    const isLarge = maxRecommendedSize && asset.size > maxRecommendedSize;
    logger.log('  ' + (isLarge ? _chalk.default.yellow(sizeLabel) : sizeLabel) + '  ' + differenceLabel + '  ' + _chalk.default.dim(asset.folder + path.sep) + _chalk.default.cyan(asset.name));
  });
  logger.log();
  if (suggestBundleSplitting) {
    logger.log();
    logger.log(_chalk.default.yellow('The bundle size is significantly larger than recommended.'));
    logger.log(_chalk.default.yellow('Consider reducing it with code splitting: https://goo.gl/9VhYWB'));
    logger.log(_chalk.default.yellow('You can also analyze the project dependencies: https://goo.gl/LeUzfb'));
  }
}
function getDifferenceLabel(currentSize, previousSize) {
  if (previousSize) {
    const difference = currentSize - previousSize;
    const fileSize = !Number.isNaN(difference) ? (0, _filesize.default)(difference) : '0';
    if (difference >= FIFTY_KILOBYTES) {
      return _chalk.default.red('(+' + fileSize + ')');
    } else if (difference < FIFTY_KILOBYTES && difference > 0) {
      return _chalk.default.yellow('(+' + fileSize + ')');
    } else if (difference < 0) {
      return _chalk.default.green('(-' + fileSize + ')');
    } else {
      return _chalk.default.dim.grey('(unchanged)');
    }
  } else {
    return _chalk.default.dim.green('(new)');
  }
}
function canReadAsset(asset) {
  return /\.(js|css)$/.test(asset) && !/service-worker\.js/.test(asset) && !/precache-manifest\.[0-9a-f]+\.js/.test(asset);
}
//# sourceMappingURL=fileSizeReporter.js.map