"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _buildStandalone = require("./buildStandalone");
var _buildPackage = require("./build-package");
var logger = _interopRequireWildcard(require("../utils/logger"));
var _actionPreflightCheck = _interopRequireDefault(require("../utils/actionPreflightCheck"));
var _getLocation = _interopRequireDefault(require("../utils/getLocation"));
var _selectWorkspaces = require("../utils/selectWorkspaces");
var _setupEnv = require("../utils/setupEnv");
var _getAllWorkspaces = require("../utils/getAllWorkspaces");
var _getModularRoot = _interopRequireDefault(require("../utils/getModularRoot"));
var _execAsync = _interopRequireDefault(require("../utils/execAsync"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
async function build({
  packagePaths,
  preserveModules = true,
  private: includePrivate,
  ancestors,
  descendants,
  changed,
  compareBranch,
  dangerouslyIgnoreCircularDependencies,
  concurrencyLevel
}) {
  const isSelective = changed || ancestors || descendants || packagePaths.length;
  const modularRoot = (0, _getModularRoot.default)();
  const [allWorkspacePackages] = await (0, _getAllWorkspaces.getAllWorkspaces)(modularRoot);

  // targets are either the set of what's specified in the selective options or all the packages in the monorepo
  const targets = isSelective ? packagePaths : [...allWorkspacePackages.keys()];
  const selectedTargets = await (0, _selectWorkspaces.selectParallellyBuildableWorkspaces)({
    targets,
    changed,
    compareBranch,
    descendants,
    ancestors,
    dangerouslyIgnoreCircularDependencies
  });
  if (!selectedTargets.length) {
    logger.log('No workspaces to build');
    process.exit(0);
  }
  logger.debug(`Building the following workspaces in order: ${JSON.stringify(selectedTargets)}`);
  for (const batch of selectedTargets) {
    logger.debug(`Building batch: ${JSON.stringify(batch)} with concurrency ${concurrencyLevel}`);
    const jobBatch = batch.map(target => {
      const packageInfo = allWorkspacePackages.get(target);
      if (!packageInfo) {
        throw new Error(`building ${target} failed - pacakge ${target} has no package info.`);
      }
      return {
        id: packageInfo.name,
        promise: () => {
          return runBuildJob({
            packageInfo,
            preserveModules,
            includePrivate,
            cwd: modularRoot
          });
        }
      };
    });
    await runBatch(jobBatch, concurrencyLevel);
  }
}
async function runBuildJob({
  packageInfo,
  preserveModules,
  includePrivate,
  cwd
}) {
  const target = packageInfo === null || packageInfo === void 0 ? void 0 : packageInfo.name;
  try {
    const targetDirectory = await (0, _getLocation.default)(target);
    await (0, _setupEnv.setupEnvForDirectory)(targetDirectory);
    if (packageInfo !== null && packageInfo !== void 0 && packageInfo.modular) {
      // If it's modular, build with Modular
      const targetType = packageInfo.modular.type;
      if (!targetType) throw new Error(`modular.type missing in ${target} package.json`);
      logger.log('\nBuilding', targetType, target);
      if (targetType === 'app' || targetType === 'esm-view') {
        await (0, _buildStandalone.buildStandalone)(target, targetType);
      } else {
        await (0, _buildPackage.buildPackage)(target, preserveModules, includePrivate);
      }
    } else {
      // Otherwise, build by running the workspace's build script
      // We're sure it's here because selectParallellyBuildableWorkspaces returns only buildable workspaces.
      logger.log('\nBuilding non-modular package', target);
      await (0, _execAsync.default)(`yarn`, ['workspace', target, 'build'], {
        cwd,
        log: false
      });
    }
  } catch (err) {
    logger.error(`building ${target} failed`);
    throw err;
  }
}
async function runBatch(jobs, concurrency) {
  while (jobs.length) {
    // Councurrency = [0|1] means no concurrency in a user-friendly way. Since 0 would spin forever, it's defaulted to 1.
    const nextConcurrentBatch = jobs.splice(0, concurrency || 1);
    const ids = nextConcurrentBatch.map(j => j.id);
    const promises = nextConcurrentBatch.map(j => j.promise);
    logger.debug(`runBatch is running a batch of length ${JSON.stringify(ids.length)}: ${JSON.stringify(ids)}`);
    // It's not Promise.all that starts the functions, but the act of invoking them, so we need to defer invocation until it's needed.
    await Promise.all(promises.map(f => f()));
  }
}
var _default = (0, _actionPreflightCheck.default)(build);
exports.default = _default;
//# sourceMappingURL=index.js.map