"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createEsmViewConfig = createEsmViewConfig;
exports.createEsmViewPluginConfig = createEsmViewPluginConfig;
var _path = _interopRequireDefault(require("path"));
var _htmlWebpackPlugin = _interopRequireDefault(require("html-webpack-plugin"));
var _dedent = _interopRequireDefault(require("dedent"));
var _fsExtra = _interopRequireDefault(require("fs-extra"));
var _parse = _interopRequireDefault(require("parse5"));
var _parsePackageName = require("../../../../utils/parsePackageName");
var _importInfo = require("../../../../utils/importInfo");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function createEsmViewConfig(importInfo, paths, isEnvProduction, useReactCreateRoot) {
  return {
    entry: !isEnvProduction ? getVirtualTrampoline(paths, useReactCreateRoot) : paths.appIndexJs,
    externals: createExternalRewriter(importInfo),
    externalsType: 'module',
    experiments: {
      outputModule: true
    },
    output: {
      module: true,
      library: {
        type: 'module'
      }
    }
  };
}
function createExternalRewriter(importInfo) {
  const {
    importSet
  } = importInfo;
  return function test({
    request
  }, callback) {
    if (request) {
      const parsedModule = (0, _parsePackageName.parsePackageName)(request);
      // If the module is absolute and it is in the import set, we want to externalise it
      if (parsedModule && parsedModule.dependencyName && importSet.has(parsedModule.dependencyName) &&
      // If this is an absolute export of css we need to deal with it in the loader
      !request.endsWith('.css')) {
        const toRewrite = (0, _importInfo.rewriteModuleSpecifier)(importInfo, request);
        return callback(undefined, toRewrite);
      }
    }
    // Otherwise we just want to bundle it
    return callback();
  };
}

// Virtual entrypoint if we're starting a ESM view - see https://github.com/webpack/webpack/issues/6437
function getVirtualTrampoline(paths, useReactCreateRoot) {
  // Build the relative path between the root and the entrypoint.
  const relativeEntrypointPath = _path.default.relative(paths.appPath, paths.appIndexJs).split(_path.default.sep).join(_path.default.posix.sep); // Separator could be win32 on Windows system, since it comes from a filesystem path. Force it to be posix since it's an URL

  const entryPointPath = `./${relativeEntrypointPath}`;
  const content = useReactCreateRoot ?
  // use the new createRoot API with React >= 18. The old one is backwards compatible, but will warn on develop.
  (0, _dedent.default)(`import { createRoot } from "react-dom/client";
            import React from "react";
            import Component from "${entryPointPath}";
            const container = document.getElementById("root");
            const root = createRoot(container);
            root.render(React.createElement(Component, null));`) :
  // use the old ReactDOM.render API with React < 18.
  (0, _dedent.default)(`import ReactDOM from "react-dom"
            import React from "react";
            import Component from "${entryPointPath}";
            const DOMRoot = document.getElementById("root");
            ReactDOM.render(React.createElement(Component, null), DOMRoot);`);
  const base64 = Buffer.from(content).toString('base64');
  return `./src/_trampoline.js!=!data:text/javascript;base64,${base64}`;
}
function createEsmViewPluginConfig(isEnvProduction, styleImports, indexPath) {
  const importUrlLinks = [];
  styleImports.forEach(importUrl => {
    importUrlLinks.push(`<link rel="stylesheet" href="${importUrl}"></script>`);
  });
  let templateContent;

  // If we have an index template in the project, let's use it
  if (indexPath) {
    templateContent = _fsExtra.default.readFileSync(indexPath, 'utf8');

    // Style (global) imports are excluded from the build, so they don't get injected. We have to manually inject them in the index template.
    if (styleImports.size) {
      const page = _parse.default.parse(templateContent);
      const html = page.childNodes.find(node => node.nodeName === 'html');
      const head = html.childNodes.find(node => node.nodeName === 'head');
      styleImports.forEach(importUrl => head.childNodes.push(..._parse.default.parseFragment(`<link rel="stylesheet" href="${importUrl}" />`).childNodes));
      templateContent = _parse.default.serialize(page);
    }
  } else {
    // Standard minimal template to start an esm-view
    templateContent = (0, _dedent.default)(`
      <!DOCTYPE html>
        <html>
          <head>
          ${importUrlLinks.join('\n')}
          </head>
          <body>
            <div id="root"></div>
          </body>
        </html>`);
  }
  return {
    plugins: [
    // We need to provide a synthetic index.html in case we're starting a ESM view
    !isEnvProduction && new _htmlWebpackPlugin.default({
      inject: true,
      templateContent,
      scriptLoading: 'module'
      // 'as unknown' used as Workaround for Typescript error:
      // 'Excessive stack depth comparing types' in current Typescript version 4.8.3
    })].filter(Boolean)
  };
}
//# sourceMappingURL=esmViewConfig.js.map