"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createPluginConfig;
var _webpack = _interopRequireDefault(require("webpack"));
var _resolve = _interopRequireDefault(require("resolve"));
var _isCi = _interopRequireDefault(require("is-ci"));
var _webpackMerge = require("webpack-merge");
var _webpackManifestPlugin = require("webpack-manifest-plugin");
var _forkTsCheckerWebpackPlugin = _interopRequireDefault(require("fork-ts-checker-webpack-plugin"));
var _ModuleNotFoundPlugin = _interopRequireDefault(require("../../plugins/ModuleNotFoundPlugin"));
var _esmViewConfig = require("./esmViewConfig");
var _appConfig = require("./appConfig");
var _developmentConfig = require("./developmentConfig");
var _productionConfig = require("./productionConfig");
var _getClientEnvironment = _interopRequireDefault(require("../../../common-scripts/getClientEnvironment"));
var _selectWorkspaces = require("../../../../utils/selectWorkspaces");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// Some apps do not need the benefits of saving a web request, so not inlining the chunk
// makes for a smoother build process.
const shouldInlineRuntimeChunk = process.env.INLINE_RUNTIME_CHUNK !== 'false';

// We will provide `paths.publicUrlOrPath` to our app
// as %PUBLIC_URL% in `index.html` and `process.env.PUBLIC_URL` in JavaScript.
// Omit trailing slash as %PUBLIC_URL%/xyz looks better than %PUBLIC_URL%xyz.
// Get environment variables to inject into our app.

async function createPluginConfig(target, isApp, isEnvProduction, shouldUseSourceMap, useTypeScript, styleImports, paths, indexPath) {
  const env = (0, _getClientEnvironment.default)(paths.publicUrlOrPath.slice(0, -1));
  const isEnvDevelopment = !isEnvProduction;

  // Typecheck package to build and descendants
  const packagesToTypecheck = await (0, _selectWorkspaces.selectWorkspaces)({
    targets: [target],
    ancestors: false,
    descendants: true,
    changed: false
  });
  const pathsToTypecheck = packagesToTypecheck.map(pkg => {
    return {
      file: `../${pkg}/src/**/*.{ts,tsx}`
    };
  });
  // For some reason the above doesn't work for the package we're building itself
  // so we add it separately
  pathsToTypecheck.push({
    file: `**/src/**/*.{ts,tsx}`
  });
  const basePlugins = {
    plugins: [
    // This gives some necessary context to module not found errors, such as
    // the requesting resource.
    new _ModuleNotFoundPlugin.default(paths.appPath),
    // Makes some environment variables available to the JS code, for example:
    // if (process.env.NODE_ENV === 'production') { ... }. See `./env.js`.
    // It is absolutely essential that NODE_ENV is set to production
    // during a production build.
    // Otherwise React will be compiled in the very slow development mode.
    new _webpack.default.DefinePlugin(env.stringified),
    // Generate an asset manifest file with the following content:
    // - "files" key: Mapping of all asset filenames to their corresponding
    //   output file so that tools can pick it up without having to parse
    //   `index.html`
    // - "entrypoints" key: Array of files which are included in `index.html`,
    //   can be used to reconstruct the HTML if necessary
    new _webpackManifestPlugin.WebpackManifestPlugin({
      fileName: 'asset-manifest.json',
      publicPath: paths.publicUrlOrPath,
      generate: (seed, files, entrypoints) => {
        const manifestFiles = files.reduce((manifest, file) => {
          manifest[file.name] = file.path;
          return manifest;
        }, seed);
        const entrypointFiles = entrypoints.main.filter(fileName => !fileName.endsWith('.map'));
        return {
          files: manifestFiles,
          entrypoints: entrypointFiles
        };
      }
      // 'as unknown' used as Workaround for Typescript error:
      // 'Excessive stack depth comparing types' in current Typescript version 4.8.3
    }),
    // Moment.js is an extremely popular library that bundles large locale files
    // by default due to how webpack interprets its code. This is a practical
    // solution that requires the user to opt into importing specific locales.
    // https://github.com/jmblog/how-to-optimize-momentjs-with-webpack
    // You can remove this if you don't use Moment.js:
    new _webpack.default.IgnorePlugin({
      resourceRegExp: /^\.\/locale$/,
      contextRegExp: /moment$/
    }),
    // TypeScript type checking turned off for CI envs
    // https://github.com/jpmorganchase/modular/issues/605
    useTypeScript && !_isCi.default && new _forkTsCheckerWebpackPlugin.default({
      async: isEnvDevelopment,
      typescript: {
        typescriptPath: _resolve.default.sync('typescript', {
          basedir: paths.appNodeModules
        }),
        configOverwrite: {
          compilerOptions: {
            sourceMap: isEnvProduction ? shouldUseSourceMap : isEnvDevelopment,
            skipLibCheck: true,
            inlineSourceMap: false,
            declarationMap: false,
            noEmit: true
          }
        },
        context: paths.appPath,
        diagnosticOptions: {
          syntactic: true,
          semantic: true
        },
        mode: 'write-references'
      },
      issue: {
        // This one is specifically to match during CI tests,
        // as micromatch doesn't match
        // '../cra-template-typescript/template/src/App.tsx'
        // otherwise.
        include: pathsToTypecheck,
        exclude: [{
          file: '**/src/**/__tests__/**'
        }, {
          file: '**/src/**/?(*.){spec|test}.*'
        }, {
          file: '**/src/setupProxy.*'
        }, {
          file: '**/src/setupTests.*'
        }]
      }
    })].filter(Boolean)
  };
  return (0, _webpackMerge.merge)([isApp ? (0, _appConfig.createAppPluginConfig)(isEnvProduction, shouldInlineRuntimeChunk, env, paths) : (0, _esmViewConfig.createEsmViewPluginConfig)(isEnvProduction, styleImports, indexPath), isEnvProduction ? (0, _productionConfig.createProductionPluginConfig)() : (0, _developmentConfig.createDevelopmentPluginConfig)(paths), basePlugins]);
}
//# sourceMappingURL=pluginConfig.js.map