"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getWebpackConfig;
var _fs = _interopRequireDefault(require("fs"));
var _chalk = _interopRequireDefault(require("chalk"));
var _webpackMerge = require("webpack-merge");
var logger = _interopRequireWildcard(require("../../../utils/logger"));
var _modules = _interopRequireDefault(require("./modules"));
var _pluginConfig = _interopRequireDefault(require("./parts/pluginConfig"));
var _esmViewConfig = require("./parts/esmViewConfig");
var _appConfig = require("./parts/appConfig");
var _developmentConfig = require("./parts/developmentConfig");
var _productionConfig = require("./parts/productionConfig");
var _baseConfig = _interopRequireDefault(require("./parts/baseConfig"));
var _config = require("../../../utils/config");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
// Source maps are resource heavy and can cause out of memory issue for large source files.

const imageInlineSizeLimit = parseInt(process.env.IMAGE_INLINE_SIZE_LIMIT || '10000');

/**
 * Generate Webpack Configuration
 * This is the production and development configuration.
 * It is focused on developer experience, fast rebuilds, and a minimal bundle.
 * @param isEnvProduction True when building, false starting
 * @param esbuildTargetFactory ES Target version
 * @param isApp True if target is an app, false if it's an ESM View
 * @param importInfo Import informations record (import set, dependencies, resolutions, template)
 * @param useReactCreateRoot True for React >= 18 as it needs a different way of instantiating rendering.
 * @param styleImports Set of Style Imports
 * @param targetPaths Relevant file paths for output
 * @returns Promise containing webpack configuration
 */
async function getWebpackConfig(target, isEnvProduction, esbuildTargetFactory, isApp, importInfo, useReactCreateRoot, styleImports, targetPaths) {
  // Check if TypeScript is setup
  const useTypeScript = _fs.default.existsSync(targetPaths.appTsConfig);
  const shouldUseSourceMap = (0, _config.getConfig)('generateSourceMap', targetPaths.appPath);

  // Variable used for enabling profiling in Production
  // passed into alias object. Uses a flag if passed into the build command
  const isEnvProductionProfile = isEnvProduction && process.argv.includes('--profile');

  // Create configurations
  const modules = await (0, _modules.default)(targetPaths);
  // base, common configuration
  const baseConfig = (0, _baseConfig.default)(isEnvProduction, isApp, targetPaths, useTypeScript, isEnvProductionProfile, imageInlineSizeLimit, modules, shouldUseSourceMap, esbuildTargetFactory, importInfo.importSet // TODO: is this needed in this form for CSS?
  );

  // Specific configuration based on modular type (app, esm-view)
  const modularTypeConfiguration = isApp ? (0, _appConfig.createAppConfig)() : (0, _esmViewConfig.createEsmViewConfig)(importInfo, targetPaths, isEnvProduction, useReactCreateRoot);

  // Specific configuration based on build type (production, development)
  const buildTypeConfiguration = isEnvProduction ? (0, _productionConfig.createProductionConfig)(shouldUseSourceMap, targetPaths) : (0, _developmentConfig.createDevelopmentConfig)();

  // If an index is provided, this is its path. Otherwise false.
  const indexPath = _fs.default.existsSync(targetPaths.appHtml) && targetPaths.appHtml;
  // Plugin configuration
  const pluginConfig = await (0, _pluginConfig.default)(target, isApp, isEnvProduction, shouldUseSourceMap, useTypeScript, styleImports, targetPaths, indexPath);

  // Merge all configurations into the final one
  const webpackConfig = (0, _webpackMerge.merge)([baseConfig, modularTypeConfiguration, buildTypeConfiguration, pluginConfig]);

  // These dependencies are so widely used for us (JPM) that it makes sense to install
  // their webpack plugin when used.
  // NOTE: this doesn't include the dependencies themselves in your app.
  const dependencyPlugins = {
    '@finos/perspective': {
      package: '@finos/perspective-webpack-plugin',
      options: {}
    },
    'react-monaco-editor': {
      package: 'monaco-editor-webpack-plugin',
      options: {
        languages: ['json', 'generic']
      }
    }
  };
  for (const [dependency, plugin] of Object.entries(dependencyPlugins)) {
    try {
      // test whether the dependency has been installed.
      // if not don't install the corresponding plugin
      require.resolve(dependency);
      try {
        require.resolve(plugin.package);
      } catch (err) {
        logger.log(`It appears you're using ${_chalk.default.cyan(dependency)}. Run ${_chalk.default.cyan.bold(`yarn add -D ${plugin.package}`)} to install`);
        throw err;
      }
      // both dependency and its webpack plugin are available, let's
      // add it to our webpack pipeline.
      const WebpackPlugin = await Promise.resolve(`${plugin.package}`).then(s => _interopRequireWildcard(require(s)));
      if (webpackConfig.plugins) {
        webpackConfig.plugins.push(new WebpackPlugin(plugin.options));
      } else {
        webpackConfig.plugins = [new WebpackPlugin(plugin.options)];
      }
    } catch (err) {
      /* silently fail */
    }
  }
  return webpackConfig;
}

/**
 * Interface to satisfy TS linting for Webpack Plugins
 */
//# sourceMappingURL=webpack.config.js.map