"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createCompiler = createCompiler;
exports.prepareProxy = prepareProxy;
var _url = _interopRequireDefault(require("url"));
var _path = _interopRequireDefault(require("path"));
var _fs = require("fs");
var _chalk = _interopRequireDefault(require("chalk"));
var _address = _interopRequireDefault(require("address"));
var _webpack = _interopRequireDefault(require("webpack"));
var _forkTsCheckerWebpackPlugin = _interopRequireDefault(require("fork-ts-checker-webpack-plugin"));
var _formatWebpackMessages = _interopRequireDefault(require("./formatWebpackMessages"));
var _logger = require("../../../utils/logger");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// Ported over functions from webpackDevServerUtils.ts

function printInstructions(appName, urls) {
  console.log();
  console.log(`You can now view ${_chalk.default.bold(appName)} in the browser.`);
  console.log();
  if (urls.lanUrlForTerminal) {
    console.log(
    // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
    `  ${_chalk.default.bold('Local:')}            ${urls.localUrlForTerminal}`);
    console.log(`  ${_chalk.default.bold('On Your Network:')}  ${urls.lanUrlForTerminal}`);
  } else {
    // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
    console.log(`  ${urls.localUrlForTerminal}`);
  }
  console.log();
  console.log('Note that the development build is not optimized.');
  console.log(`To create a production build, use ${_chalk.default.cyan('yarn')} build.`);
  console.log();
}
function createCompiler(appName, webpackConfig, urls, useTypeScript) {
  // "Compiler" is a low-level interface to webpack.
  // It lets us listen to some events and provide our own custom messages.
  const compiler = (0, _webpack.default)(webpackConfig);

  // "invalid" event fires when you have changed a file, and webpack is
  // recompiling a bundle. WebpackDevServer takes care to pause serving the
  // bundle, so if you refresh, it'll wait instead of serving the old one.
  // "invalid" is short for "bundle invalidated", it doesn't imply any errors.
  compiler.hooks.invalid.tap('invalid', () => {
    console.log('Compiling...');
  });
  let isFirstCompile = true;
  if (useTypeScript) {
    _forkTsCheckerWebpackPlugin.default.getCompilerHooks(compiler).waiting.tap('awaitingTypeScriptCheck', () => {
      console.log(_chalk.default.yellow('Files successfully emitted, waiting for typecheck results...'));
    });
  }

  // "done" event fires when webpack has finished recompiling the bundle.
  // Whether or not you have warnings or errors, you will get this event.
  compiler.hooks.done.tap('done', stats => {
    // We have switched off the default webpack output in WebpackDevServer
    // options so we are going to "massage" the warnings and errors and present
    // them in a readable focused way.
    // We only construct the warnings and errors for speed:
    // https://github.com/facebook/create-react-app/issues/4492#issuecomment-421959548
    const statsData = stats.toJson({
      all: false,
      warnings: true,
      errors: true
    });
    const messages = (0, _formatWebpackMessages.default)({
      warnings: statsData.warnings,
      errors: statsData.errors
    });
    const isSuccessful = !messages.errors.length && !messages.warnings.length;
    if (isSuccessful) {
      console.log(_chalk.default.green('Compiled successfully!'));
    }
    if (isSuccessful && isFirstCompile) {
      printInstructions(appName, urls);
    }
    isFirstCompile = false;

    // If errors exist, only show errors.
    if (messages.errors.length) {
      // Only keep the first error. Others are often indicative
      // of the same problem, but confuse the reader with noise.
      if (messages.errors.length > 1) {
        messages.errors.length = 1;
      }
      console.log(_chalk.default.red('Failed to compile.\n'));
      console.log(messages.errors.join('\n\n'));
      return;
    }

    // Show warnings if no errors were found.
    if (messages.warnings.length) {
      console.log(_chalk.default.yellow('Compiled with warnings.\n'));
      console.log(messages.warnings.join('\n\n'));

      // Teach some ESLint tricks.
      console.log('\nSearch for the ' + _chalk.default.underline(_chalk.default.yellow('keywords')) + ' to learn more about each warning.');
      console.log('To ignore, add ' + _chalk.default.cyan('// eslint-disable-next-line') + ' to the line before.\n');
    }
  });

  // You can safely remove this after ejecting.
  // We only use this block for testing of Create React App itself:
  const isSmokeTest = process.argv.some(arg => arg.indexOf('--smoke-test') > -1);
  if (isSmokeTest) {
    compiler.hooks.failed.tap('smokeTest', () => {
      process.exit(1);
    });
    compiler.hooks.done.tap('smokeTest', stats => {
      if (stats.hasErrors() || stats.hasWarnings()) {
        process.exit(1);
      } else {
        process.exit(0);
      }
    });
  }
  return compiler;
}
function prepareProxy(proxy, appPublicFolder, servedPathname) {
  // `proxy` lets you specify alternate servers for specific requests.
  if (!proxy) {
    return undefined;
  }
  if (typeof proxy !== 'string') {
    (0, _logger.log)(_chalk.default.red('When specified, "proxy" in package.json must be a string.'));
    (0, _logger.log)(_chalk.default.red('Instead, the type of "proxy" was "' + typeof proxy + '".'));
    (0, _logger.log)(_chalk.default.red('Either remove "proxy" from package.json, or make it a string.'));
    process.exit(1);
  }

  // If proxy is specified, let it handle any request except for
  // files in the public folder and requests to the WebpackDevServer socket endpoint.
  // https://github.com/facebook/create-react-app/issues/6720
  const sockPath = process.env.WDS_SOCKET_PATH || '/ws';
  const isDefaultSockHost = !process.env.WDS_SOCKET_HOST;
  function mayProxy(pathname) {
    const maybePublicPath = _path.default.resolve(appPublicFolder, pathname.replace(new RegExp('^' + servedPathname), ''));
    const isPublicFileRequest = (0, _fs.existsSync)(maybePublicPath);
    // used by webpackHotDevClient
    const isWdsEndpointRequest = isDefaultSockHost && pathname.startsWith(sockPath);
    return !(isPublicFileRequest || isWdsEndpointRequest);
  }
  if (!/^http(s)?:\/\//.test(proxy)) {
    console.log(_chalk.default.red('When "proxy" is specified in package.json it must start with either http:// or https://'));
    process.exit(1);
  }
  const target = process.platform === 'win32' ? resolveLoopback(proxy) : proxy;
  return [{
    target,
    logLevel: 'silent',
    // For single page apps, we generally want to fallback to /index.html.
    // However we also want to respect `proxy` for API calls.
    // So if `proxy` is specified as a string, we need to decide which fallback to use.
    // We use a heuristic: We want to proxy all the requests that are not meant
    // for static assets and as all the requests for static assets will be using
    // `GET` method, we can proxy all non-`GET` requests.
    // For `GET` requests, if request `accept`s text/html, we pick /index.html.
    // Modern browsers include text/html into `accept` header when navigating.
    // However API calls like `fetch()` won’t generally accept text/html.
    // If this heuristic doesn’t work well for you, use `src/setupProxy.js`.
    context: function (pathname, req) {
      const shouldFilter = req.method !== 'GET' || mayProxy(pathname) && req.headers.accept && req.headers.accept.indexOf('text/html') === -1 ? true : false;
      return shouldFilter;
    },
    onProxyReq: function (proxyReq, _req, _res, _options) {
      // Browsers may send Origin headers even with same-origin
      // requests. To prevent CORS issues, we have to change
      // the Origin to match the target URL.
      if (proxyReq.getHeader('origin')) {
        proxyReq.setHeader('origin', target);
      }
    },
    onError: onProxyError(target),
    secure: false,
    changeOrigin: true,
    ws: true,
    xfwd: true
  }];
}
function resolveLoopback(proxy) {
  const o = _url.default.parse(proxy);
  if (o.hostname !== 'localhost') {
    return proxy;
  }
  // Unfortunately, many languages (unlike node) do not yet support IPv6.
  // This means even though localhost resolves to ::1, the application
  // must fall back to IPv4 (on 127.0.0.1).
  // We can re-enable this in a few years.
  /*try {
      o.hostname = address.ipv6() ? '::1' : '127.0.0.1';
    } catch (_ignored) {
      o.hostname = '127.0.0.1';
    }*/

  try {
    // Check if we're on a network; if we are, chances are we can resolve
    // localhost. Otherwise, we can just be safe and assume localhost is
    // IPv4 for maximum compatibility.
    if (!_address.default.ip()) {
      o.hostname = '127.0.0.1';
    }
  } catch (_ignored) {
    o.hostname = '127.0.0.1';
  }
  return _url.default.format(o);
}

// We need to provide a custom onError function for httpProxyMiddleware.
// It allows us to log custom error messages on the console.
function onProxyError(proxy) {
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  return (err, req, res) => {
    const host = req.headers && req.headers.host;
    console.log(_chalk.default.red('Proxy error:') + ' Could not proxy request ' + _chalk.default.cyan(req.url) + ' from ' + _chalk.default.cyan(host) + ' to ' + _chalk.default.cyan(proxy) + '.');
    console.log('See https://nodejs.org/api/errors.html#errors_common_system_errors for more information (' +
    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
    _chalk.default.cyan(err.code) + ').');
    console.log();

    // And immediately send the proper error response to the client.
    // Otherwise, the request will eventually timeout with ERR_EMPTY_RESPONSE on the client side.
    if (res.writeHead && !res.headersSent) {
      res.writeHead(500);
    }
    res.end(
    // eslint-disable-next-line @typescript-eslint/restrict-plus-operands
    'Proxy error: Could not proxy request ' + req.url + ' from ' + host + ' to ' + proxy + ' (' +
    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
    err.code + ').');
  };
}
//# sourceMappingURL=webpackDevServerUtils.js.map