"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var path = _interopRequireWildcard(require("path"));
var _actionPreflightCheck = _interopRequireDefault(require("./utils/actionPreflightCheck"));
var _resolveAsBin = require("./utils/resolveAsBin");
var _getModularRoot = _interopRequireDefault(require("./utils/getModularRoot"));
var _execAsync = _interopRequireDefault(require("./utils/execAsync"));
var _gitActions = require("./utils/gitActions");
var logger = _interopRequireWildcard(require("./utils/logger"));
var _utils = require("./test/utils");
var _getAllWorkspaces = require("./utils/getAllWorkspaces");
var _selectWorkspaces = require("./utils/selectWorkspaces");
var _unobtrusiveModular = require("./utils/unobtrusiveModular");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
async function lint(options, packages = []) {
  var _lintRegexes;
  const {
    fix = false,
    staged = false,
    regex: userRegexes = [],
    ancestors = false,
    descendants = false,
    changed = false,
    compareBranch,
    diff = false
  } = options;
  const modularRoot = (0, _getModularRoot.default)();
  const lintExtensions = ['.ts', '.tsx', '.js', '.jsx'];
  const [workspaceMap] = await (0, _getAllWorkspaces.getAllWorkspaces)();
  const isSelective = changed || ancestors || descendants || packages.length;
  if ((staged || diff || userRegexes.length) && isSelective) {
    logger.error("Can't specify --staged, --diff or --regex along with selective options");
    process.exit(-1);
  }
  if (staged && diff || staged && userRegexes.length || diff && userRegexes.length) {
    logger.error("--regex, --diff and --staged are mutually exclusive options that can't be used together");
    process.exit(-1);
  }
  let modularTargets;
  let nonModularTargets;

  // If --diff or --staged, lint only diffed or staged files,
  // otherwise lint selected packages or all packages if nothing selected
  let lintRegexes;
  if (diff || staged) {
    // Calculate the file regexes of --diff or --staged
    try {
      // This can throw in case there is no git directory; in this case, we need to continue like nothing happened
      const diffedFiles = staged ? (0, _gitActions.getStagedFiles)() : (0, _gitActions.getDiffedFiles)(compareBranch);
      if (diffedFiles.length === 0) {
        logger.warn(`No staged or diffed files detected.`);
        process.exit(0);
      } else {
        lintRegexes = diffedFiles.filter(p => lintExtensions.includes(path.extname(p))).map(p => `<rootDir>/${p}`);

        // if none of the diffed files meet the extension criteria, do not lint
        // end the process early with a success
        if (!lintRegexes.length) {
          logger.warn('No lintable diffed or staged target files found');
          process.exit(0);
        }
      }
    } catch {
      logger.error('Getting staged or diffed files failed - are you sure this is a git repo?');
      process.exit(1);
    }
    nonModularTargets = [];
  } else if (userRegexes.length) {
    // Don't calculate any regexes or non-modular targets - only use the user provided regexes
    lintRegexes = [];
    nonModularTargets = [];
  } else {
    // Compute selected packages or run on all modular & non-modular packages
    let selectedTargets;
    if (isSelective) {
      selectedTargets = await (0, _selectWorkspaces.selectWorkspaces)({
        targets: packages,
        changed,
        compareBranch,
        descendants,
        ancestors
      });
    } else {
      selectedTargets = [...workspaceMap.keys()];
    }

    // Split packages into modular and non-modular testable. Make sure that "root" is not there.
    [modularTargets, nonModularTargets] = (0, _unobtrusiveModular.partitionPackages)(selectedTargets, workspaceMap, 'lint');

    // Compute patterns to pass Jest for the packages that we want to test
    const selectedPackageRegexes = await (0, _unobtrusiveModular.computeRegexesFromPackageNames)(modularTargets);
    const processedPackageRegexes = selectedPackageRegexes.map(regex => `<rootDir>/${regex}/src/**/*.{js,jsx,ts,tsx}`);
    lintRegexes = processedPackageRegexes;
  }

  // If we computed no regexes and there are no non-modular packages to lint, bail out
  if (!((_lintRegexes = lintRegexes) !== null && _lintRegexes !== void 0 && _lintRegexes.length) && !userRegexes && !nonModularTargets.length) {
    process.stdout.write('No workspaces found in selection\n');
    process.exit(0);
  }
  const jestEslintConfig = {
    runner: require.resolve('modular-scripts/jest-runner-eslint'),
    displayName: 'lint',
    rootDir: modularRoot,
    testMatch: lintRegexes,
    testPathIgnorePatterns: ['/node_modules/', '/dist/']
  };
  const lintArgs = [...userRegexes, '--config', (0, _utils.generateJestConfig)(jestEslintConfig)];
  const lintBin = await (0, _resolveAsBin.resolveAsBin)('jest-cli');
  logger.debug(`Regexes for Modular packages being linted: ${JSON.stringify(lintRegexes)}`);
  logger.debug(`User provided regexes: ${JSON.stringify(userRegexes)}`);

  // Lint modular packages
  if (lintRegexes.length || userRegexes.length) {
    try {
      logger.debug(`Running ${lintBin} with cwd: ${(0, _getModularRoot.default)()} and args: ${JSON.stringify(lintArgs)}`);
      await (0, _execAsync.default)(lintBin, lintArgs, {
        cwd: modularRoot,
        log: false,
        // @ts-ignore
        env: {
          MODULAR_ROOT: modularRoot,
          MODULAR_LINT_FIX: String(fix)
        }
      });
      if (staged && fix) {
        lintRegexes = lintRegexes.map(p => p.replace('<rootDir>/', ''));
        (0, _gitActions.addFiles)(lintRegexes);
      }
    } catch (err) {
      logger.debug(err.message);
      // ✕ Modular lint did not pass
      throw new Error('\u2715 Modular lint did not pass');
    }
  }
  if (nonModularTargets.length) {
    try {
      logger.debug(`Running lint command in the following non-modular packages: ${JSON.stringify(nonModularTargets)}`);
      for (const target of nonModularTargets) {
        await (0, _execAsync.default)(`yarn`, ['workspace', target, 'lint'], {
          cwd: (0, _getModularRoot.default)(),
          log: false
        });
      }
    } catch (err) {
      logger.debug(err.message);
      // ✕ Modular lint did not pass
      throw new Error('\u2715 Modular lint did not pass');
    }
  }
}
var _default = (0, _actionPreflightCheck.default)(lint);
exports.default = _default;
//# sourceMappingURL=lint.js.map