"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addPackageForTests = addPackageForTests;
exports.buildPackageForTests = buildPackageForTests;
exports.mockPreflightImplementation = void 0;
exports.mockProcessExit = mockProcessExit;
exports.mockProcessLogs = mockProcessLogs;
exports.runLintForTests = runLintForTests;
exports.runSelectForTests = runSelectForTests;
exports.runTestForTests = runTestForTests;
exports.setupMocks = setupMocks;
var _path = _interopRequireDefault(require("path"));
var _fsExtra = _interopRequireDefault(require("fs-extra"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
/**
 * Adds a package to the current modular repo
 * Can't be made compatible with path parameter as addPackage has a check that uses path.relative()
 * that will fail as it uses the CWD
 * Use with setupMocks/Mocked getModularRoot to ensure that it adds the package in the correct test context
 * @param name Name of package to add
 * @param type Modular Type of package being added (template)
 */
async function addPackageForTests(name, type) {
  const {
    default: addPackage
  } = await Promise.resolve().then(() => _interopRequireWildcard(require('../addPackage')));
  await addPackage({
    name,
    type
  });
}
/**
 * Runs tests based on the provided parameters & returns spied process.exit and any errors thrown
 * Mocking process.exit ensures the test function can't call process.exit and end the process for all tests
 * Use with setupMocks/Mocked getModularRoot to ensure that it adds the package in the correct test context
 */
async function runTestForTests(options) {
  const mockExit = mockProcessExit();
  const {
    default: test
  } = await Promise.resolve().then(() => _interopRequireWildcard(require('./index')));
  // Mock process.exit so that it doesn't do anything during these tests
  let err;
  try {
    await test({
      env: 'jsdom',
      regex: options.regex,
      changed: options.changed
    }, options.packages);
  } catch (e) {
    err = e.message;
  }
  // Tests can fail in two ways: error, and process.exit - we have to cover both
  return {
    mockExit,
    err
  };
}
/**
 * Runs select command with provided options & returns log output
 * Use with setupMocks/Mocked getModularRoot to ensure that it adds the package in the correct test context
 */
async function runSelectForTests({
  changed = false,
  compareBranch = undefined,
  ancestors = false,
  descendants = false,
  buildable = false,
  selectedPackages = []
}) {
  const stdoutArray = [];
  const stderrArray = [];
  mockProcessLogs(stdoutArray, stderrArray);
  const {
    default: select
  } = await Promise.resolve().then(() => _interopRequireWildcard(require('../select')));
  await select({
    changed,
    compareBranch,
    ancestors,
    descendants,
    buildable,
    selectedPackages
  });
  return {
    stdout: stdoutArray.join('\n'),
    stderr: stderrArray.join('\n')
  };
}
/**
 * Runs lint command with provided options & returns spied process.exit, log output, and any errors thrown
 * Use with setupMocks/Mocked getModularRoot to ensure that it adds the package in the correct test context
 */
async function runLintForTests({
  all = false,
  fix = false,
  staged = false,
  packages = [],
  ancestors = false,
  descendants = false,
  changed = false,
  compareBranch
}, regexes = []) {
  const stdoutArray = [];
  const stderrArray = [];
  mockProcessLogs(stdoutArray, stderrArray);
  const {
    default: lint
  } = await Promise.resolve().then(() => _interopRequireWildcard(require('../lint')));
  await lint({
    all,
    fix,
    staged,
    packages,
    ancestors,
    descendants,
    changed,
    compareBranch
  }, regexes);
  return {
    stderrArray,
    stdoutArray
  };
}

/**
 * Build a specified package
 * Use with setupMocks/Mocked getModularRoot to ensure that it looks for the package in the correct test context
 * @param targetPackage Target package to build
 * @param config Optional array of configuration options and their values to affect build
 */
async function buildPackageForTests(targetPackage, config) {
  if (config) await writeConfig(targetPackage, config);
  const {
    default: build
  } = await Promise.resolve().then(() => _interopRequireWildcard(require('../build-scripts/index')));
  await build({
    packagePaths: [targetPackage],
    preserveModules: false,
    private: false,
    ancestors: false,
    descendants: false,
    changed: false,
    dangerouslyIgnoreCircularDependencies: false
  });
  if (config) await deleteConfig(targetPackage);
}

/**
 * Write a modular configuration file in the temporary
 * modular repo to configure modular command behaviour
 * Use with setupMocks/Mocked getModularRoot to ensure that it looks for the package in the correct test context
 * @param targetPackage name of package being configured
 * @param config Array of configuration options and their value
 */
async function writeConfig(targetPackage, config) {
  const {
    default: getWorkspaceLocation
  } = await Promise.resolve().then(() => _interopRequireWildcard(require('../utils/getLocation')));
  const targetPath = await getWorkspaceLocation(targetPackage);
  await _fsExtra.default.writeFile(_path.default.join(targetPath, '.modular.js'), `module.exports = {\n
        ${config.join(',\n')},\n};`);
}

/**
 * Opposite of write config - doesn't actually delete, just overwrites with an empty config
 * Use with setupMocks/Mocked getModularRoot to ensure that it looks for the package in the correct test context
 * @param targetPackage name of package being configured
 */
async function deleteConfig(targetPackage) {
  const {
    default: getWorkspaceLocation
  } = await Promise.resolve().then(() => _interopRequireWildcard(require('../utils/getLocation')));
  const targetPath = await getWorkspaceLocation(targetPackage);
  // Can't actually delete due to permission issues so just overwrite with empty
  await _fsExtra.default.writeFile(_path.default.join(targetPath, '.modular.js'), `module.exports = {};`);
}

/**
 * Set up mocked ModularRoot and skip preflight checks so that dynamically imported code can run
 * on the temporary modular repo provided, rather than on the current modular root
 * @param modularRoot Path to temporary modular repo to mock getModularRoot to
 */
function setupMocks(modularRoot) {
  // Resets mocks and modules so that modules use updated mocked modularRoot
  jest.resetAllMocks();
  jest.resetModules();
  // Mock the modular root per temporary modular repo
  jest.doMock('../utils/getModularRoot', () => {
    return {
      __esModule: true,
      default: () => modularRoot
    };
  });
  // Skip preflight in tests (faster, avoids the need to mock getModularRoot statically)
  jest.doMock('../utils/actionPreflightCheck', () => mockPreflightImplementation);
}

/**
 * Mocks process.exit so that it doesn't happen during tests
 * @returns jest.SpyInstance which you can use to test with expect(mockExit).toHaveBeenCalled()
 */
function mockProcessExit() {
  return jest.spyOn(process, 'exit').mockImplementation(() => {
    return undefined;
  });
}

/**
 * Mocks process.stdout so that it doesn't happen during tests
 * @returns jest.SpyInstance which you can use to test with expect(mockExit).toHaveBeenCalledWith('Log that you're expecting')
 */
function mockProcessLogs(stdout, stderr) {
  jest.spyOn(process.stdout, 'write').mockImplementation(text => {
    stdout.push(String(text));
    return undefined;
  });
  jest.spyOn(process.stderr, 'write').mockImplementation(text => {
    stderr.push(String(text));
    return undefined;
  });
  jest.spyOn(console, 'log').mockImplementation(text => {
    stdout.push(String(text));
    return undefined;
  });
}
const mockPreflightImplementation = {
  __esModule: true,
  default: fn => {
    return fn;
  }
};
exports.mockPreflightImplementation = mockPreflightImplementation;
//# sourceMappingURL=mockFunctions.js.map