"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getConfig = getConfig;
var _path = _interopRequireDefault(require("path"));
var _cosmiconfig = require("cosmiconfig");
var _getModularRoot = _interopRequireDefault(require("./getModularRoot"));
var logger = _interopRequireWildcard(require("./logger"));
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const modularRoot = (0, _getModularRoot.default)();

// Where cosmiconfig can look for the configuration
const searchPlaces = ['.modular.js', 'package.json', `.modularrc`, `.modularrc.json`, `.modularrc.yaml`, `.modularrc.yml`, `.modularrc.js`, `.modularrc.cjs`, `modular.config.js`, `modular.config.cjs`];

/**
 * Configuration file interface
 */

/**
 * Defaults and env variable overrides
 */
const defs = {
  useModularEsbuild: {
    default: false,
    override: process.env.USE_MODULAR_ESBUILD === 'true' || process.env.USE_MODULAR_ESBUILD === 'false' ? process.env.USE_MODULAR_ESBUILD === 'true' : undefined
  },
  externalCdnTemplate: {
    default: 'https://esm.sh/[name]@[version]',
    override: process.env.EXTERNAL_CDN_TEMPLATE
  },
  externalBlockList: {
    default: [],
    override: process.env.EXTERNAL_BLOCK_LIST ? process.env.EXTERNAL_BLOCK_LIST.split(',') : undefined
  },
  externalAllowList: {
    default: ['**'],
    override: process.env.EXTERNAL_ALLOW_LIST ? process.env.EXTERNAL_ALLOW_LIST.split(',') : undefined
  },
  publicUrl: {
    default: '',
    override: process.env.PUBLIC_URL
  },
  generateSourceMap: {
    default: true,
    override: process.env.GENERATE_SOURCEMAP === 'true' || process.env.GENERATE_SOURCEMAP === 'false' ? process.env.GENERATE_SOURCEMAP === 'true' : undefined
  },
  swcJest: {
    default: false,
    override: process.env.SWC_JEST === 'true' || process.env.SWC_JEST === 'false' ? process.env.SWC_JEST === 'true' : undefined
  }
};
const explorer = (0, _cosmiconfig.cosmiconfigSync)('modular', {
  searchPlaces
});
const configurations = new Map();

/**
 * Searches for configuration files in a given workspace,
 * first checks if there's an already loaded & cached configuration, otherwise
 * runs a cosmiconfig search and saves it for future use
 * @param workspacePath Location to search
 * @returns cosmiconfig configuration, if any found
 */
function searchConfig(workspacePath) {
  const cachedConfig = configurations.get(workspacePath);
  if (cachedConfig !== undefined) {
    return cachedConfig;
  } else {
    const config = explorer.search(_path.default.join(workspacePath, 'package.json'));
    configurations.set(workspacePath, config);
    return config;
  }
}

/**
 * Get the configured value in a workspace for a given configuration field.
 * @param field Field containing the configuration variable to read
 * @param workspacePath Path of workspace to which configuration applies
 * @returns configured value:
 * - the override environment variable if configured
 * - the value stated in the package's config file if provided
 * - the value stated in the root config file if provided
 * - the default value if neither environment variable nor the config file are provided
 */
function getConfig(field, workspacePath) {
  const configResult = searchConfig(workspacePath);
  const configValue = configResult ? configResult.config[field] : undefined;
  const rootConfigResult = searchConfig(modularRoot);
  const rootConfigValue = rootConfigResult ? rootConfigResult.config[field] : undefined;
  const returnConfigValue = defs[field].override ?? configValue ?? rootConfigValue ?? defs[field].default;
  if (defs[field].override !== undefined) {
    logger.debug(`${field} configured to ${JSON.stringify(returnConfigValue)} by the environment variable provided`);
  } else if (configValue !== undefined) {
    logger.debug(`${field} configured to ${JSON.stringify(returnConfigValue)} through package's modular configuration at ${workspacePath}`);
  } else if (rootConfigValue !== undefined) {
    logger.debug(`${field} configured to ${JSON.stringify(returnConfigValue)} through the project's root modular configuration`);
  } else {
    logger.debug(`Using default configuration for ${field}: ${JSON.stringify(returnConfigValue)}`);
  }
  return returnConfigValue;
}
//# sourceMappingURL=config.js.map