"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getPackageDependencies = getPackageDependencies;
exports.parseYarnLock = parseYarnLock;
exports.resolvePackageDependencies = resolvePackageDependencies;
var path = _interopRequireWildcard(require("path"));
var fs = _interopRequireWildcard(require("fs-extra"));
var _tsMorph = require("ts-morph");
var lockfile = _interopRequireWildcard(require("@yarnpkg/lockfile"));
var yaml = _interopRequireWildcard(require("js-yaml"));
var _getModularRoot = _interopRequireDefault(require("./getModularRoot"));
var _getLocation = _interopRequireDefault(require("./getLocation"));
var _getWorkspaceInfo = _interopRequireDefault(require("./getWorkspaceInfo"));
var _parsePackageName = require("./parsePackageName");
var logger = _interopRequireWildcard(require("./logger"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
/* Get dependencies from import / require declarations, since they could be hoisted to the root workspace. Exclude test files. */
function analyzeDependencies(workspaceLocation) {
  const project = new _tsMorph.Project();
  project.addSourceFilesAtPaths(path.join(workspaceLocation, 'src/**/!(*.test){.d.ts,.ts,.js,.jsx,.tsx}'));
  const dependencySet = new Set();
  const rawImportSet = new Set();
  project.getSourceFiles().forEach(sourceFile => sourceFile.getImportDeclarations().forEach(declaration => {
    const moduleSpecifier = declaration.getModuleSpecifierValue();
    const {
      dependencyName
    } = (0, _parsePackageName.parsePackageName)(moduleSpecifier);
    if (dependencyName) {
      dependencySet.add(dependencyName);
      rawImportSet.add(moduleSpecifier);
    }
  }));
  return {
    dependencies: Array.from(dependencySet),
    rawImports: rawImportSet
  };
}
async function getPackageDependencies(target) {
  // This function is based on the assumption that nested package are not supported, so dependencies can be either declared in the
  // target's package.json or hoisted up to the workspace root.
  const targetLocation = await (0, _getLocation.default)(target);
  const workspaceInfo = await (0, _getWorkspaceInfo.default)();
  const rootManifest = fs.readJSONSync(path.join((0, _getModularRoot.default)(), 'package.json'));
  const targetManifest = fs.readJSONSync(path.join(targetLocation, 'package.json'));
  const lockFileContents = fs.readFileSync(path.join((0, _getModularRoot.default)(), 'yarn.lock'), {
    encoding: 'utf8',
    flag: 'r'
  });

  // Selective CDN resolutions is a list of dependencies that we want our CDN to use to build our dependencies with.
  // Some CDNs support this mechanism - https://github.com/esm-dev/esm.sh#specify-external-dependencies
  // This is especially useful if we have stateful dependencies (like React) that we need to query the same version through all our CDN depenencies
  // We just output them as a comma-separated parameter in the CDN template as [selectiveCDNResolutions]
  const selectiveCDNResolutions = (targetManifest === null || targetManifest === void 0 ? void 0 : targetManifest.resolutions) ?? {};

  // Package dependencies can be either local to the package or in the root package (hoisted)
  const packageDeps = Object.assign(Object.create(null), rootManifest.devDependencies, rootManifest.dependencies, targetManifest.devDependencies, targetManifest.dependencies);
  const lockDeps = parseYarnLock(lockFileContents, packageDeps);
  const dependenciesfromSource = analyzeDependencies(targetLocation);
  const resolvedPackageDependencies = resolvePackageDependencies({
    dependencies: dependenciesfromSource.dependencies,
    packageDeps,
    lockDeps,
    workspaceInfo
  });

  // Log the errors
  resolvedPackageDependencies.manifestMiss.forEach(depName => logger.error(`Package ${depName} imported in ${target} source but not found in package dependencies or hoisted dependencies - this will prevent you from successfully building, starting or moving esm-views and will cause an error in the next release of modular`));
  resolvedPackageDependencies.lockFileMiss.forEach(depName => logger.error(`Package ${depName} imported in ${target} source but not found in lockfile - this will prevent you from successfully building, starting or moving esm-views and will cause an error in the next release of modular. Have you installed your dependencies?`));

  // Return resolutions, omitting the errors
  return {
    selectiveCDNResolutions,
    manifest: resolvedPackageDependencies.manifest,
    resolutions: resolvedPackageDependencies.resolutions,
    rawImports: dependenciesfromSource.rawImports
  };
}
function parseYarnLock(lockFile, deps) {
  return parseYarnLockV1(lockFile, deps) || parseYarnLockV3(lockFile, deps);
}
function resolvePackageDependencies({
  dependencies,
  packageDeps,
  lockDeps,
  workspaceInfo
}) {
  /* Get dependencies from package.json or pinned version in lockfile (resolution)
   * Exclude workspace dependencies. Warn if a dependency is imported in the source code
   * but not specified in any of the package.jsons
   */
  const accumulator = {
    manifest: {},
    resolutions: {},
    manifestMiss: [],
    lockFileMiss: []
  };
  return dependencies.reduce((acc, depName) => {
    var _workspaceInfo$depNam;
    const depManifestVersion = packageDeps[depName];
    if (depManifestVersion) {
      acc.manifest[depName] = depManifestVersion;
    } else {
      acc.manifestMiss.push(depName);
    }
    // Resolve either from lockfile or from workspace info. Precedence is given to lockfile.
    const resolutionVersion = lockDeps[depName] ?? ((_workspaceInfo$depNam = workspaceInfo[depName]) === null || _workspaceInfo$depNam === void 0 ? void 0 : _workspaceInfo$depNam.version);
    if (resolutionVersion) {
      acc.resolutions[depName] = resolutionVersion;
    } else {
      acc.lockFileMiss.push(depName);
    }
    return acc;
  }, accumulator);
}
function parseYarnLockV1(lockFileContents, deps) {
  let parsedLockfile;
  try {
    parsedLockfile = lockfile.parse(lockFileContents);
  } catch (e) {
    return null;
  }
  const lockFileEntries = parsedLockfile.object;
  return Object.entries(deps).reduce((acc, [name, version]) => {
    var _lockFileEntries;
    acc[name] = (_lockFileEntries = lockFileEntries[`${name}@${version}`]) === null || _lockFileEntries === void 0 ? void 0 : _lockFileEntries.version;
    return acc;
  }, {});
}
function parseYarnLockV3(lockFileContents, deps) {
  const dependencyArray = Object.entries(deps);
  const npmDependencyList = new Map(dependencyArray.map(([name, version]) => [`${name}@npm:${version}`, name]));
  const lockFileEntries = yaml.load(lockFileContents);
  // This function loops over all the dependency ranges listed in the lockfile and tries to match the given dependencies with an exact version.
  return Object.entries(lockFileEntries).reduce((acc, [name, {
    version
  }]) => {
    // Yarn v3 lockfile comes with keys like "'yargs@npm:^15.0.2, yargs@npm:^15.1.0, yargs@npm:^15.3.1, yargs@npm:^15.4.1'" - split them
    const entryDependencies = name.split(', ');
    entryDependencies.some(dep => {
      const npmName = npmDependencyList.get(dep);
      return !!(npmName && (acc[npmName] = version));
    });
    return acc;
  }, {});
}
//# sourceMappingURL=getPackageDependencies.js.map